// Demo: Dual CAN-BUS Shields, Data Pass-through
// -----------------------------------------------------------------------------
// Reference: Cory J. Fowler
// January 31st 2014
// This examples the ability of this library to support more than one MCP2515 
// based CAN interface.
// https://github.com/coryjfowler/MCP_CAN_lib/blob/master/examples/Dual_CAN/Dual_CAN.ino
//
// Modified by Copperhill Technologies (2023) for the use with 
// the espBerry and RPi CAN HATs.
//
#include <Arduino.h>
#include "mcp2515.h"

// CAN Ports Functions
// -----------------------------------------------------------------------------
extern byte canInit(const int canPort, const CAN_SPEED canSpeed, 
                    const int canOpMode);
extern byte canRx(byte cPort, unsigned long* lMsgID, byte* cMessageIDFormat, 
                  byte* cData, byte* cDataLen);
extern byte canTx(byte cPort, unsigned long lMsgID, byte cMessageIDFormat, 
                  byte* cData, byte cDataLen);
// -----------------------------------------------------------------------------
#define CAN0BR      CAN_500KBPS
#define CAN1BR      CAN_250KBPS
// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------
void setup()
{
  // Initialize Serial Monitor
  Serial.begin(115200);

  // Initialize the CAN ports
  if(canInit(0, CAN0BR, CAN_OP_NORMAL) == CAN_OK)
      Serial.println("CAN0 Initialized.");
  else
      Serial.println("CAN0 Initialization Error.");

  if(canInit(1, CAN1BR, CAN_OP_NORMAL) == CAN_OK)
      Serial.println("CAN1 Initialized.");
  else
      Serial.println("CAN1 Initialization Error.");

}// end setup

// -----------------------------------------------------------------------------
// Loop
// -----------------------------------------------------------------------------
void loop()
{
  // Declarations
  unsigned long lMsgID;
  byte bExtFormat;
  byte cData[8];
  byte cDataLen;

  // Check for data at CAN port 0
  if(canRx(0, &lMsgID, &bExtFormat, cData, &cDataLen) == CAN_OK)
  {
      if(canTx(1, lMsgID, bExtFormat, cData, cDataLen) != CAN_OK)
        Serial.println("CAN1: Transmit Error.");

  }// end if

  // Check for data at CAN port 1
  if(canRx(1, &lMsgID, &bExtFormat, cData, &cDataLen) == CAN_OK)
  {
      if(canTx(0, lMsgID, bExtFormat, cData, cDataLen) != CAN_OK)
        Serial.println("CAN0: Transmit Error.");

  }// end if

}// end loop

// -----------------------------------------------------------------------------
// END of FILE
// -----------------------------------------------------------------------------
